@@ -9,18 +9,19 @@ import logging
 from urllib.parse import parse_qs, urlparse
 import uuid
 
-from cryptography.exceptions import InvalidTag
+from cryptography.exceptions import InvalidSignature, InvalidTag
+from cryptography.hazmat.primitives import serialization
+from cryptography.hazmat.primitives.asymmetric import ed25519
+from cryptography.hazmat.primitives.asymmetric import x25519
 from cryptography.hazmat.primitives.ciphers.aead import ChaCha20Poly1305
-import curve25519
-import ed25519
 
+from pyhap import tlv
 from pyhap.const import (
     CATEGORY_BRIDGE,
     HAP_REPR_CHARS,
     HAP_REPR_STATUS,
     HAP_SERVER_STATUS,
 )
-import pyhap.tlv as tlv
 from pyhap.util import long_to_bytes
 
 from .hap_crypto import hap_hkdf, pad_tls_nonce
@@ -368,11 +369,11 @@ class HAPServerHandler:
         )
 
         data = output_key + client_username + client_ltpk
-        verifying_key = ed25519.VerifyingKey(client_ltpk)
+        verifying_key = ed25519.Ed25519PublicKey.from_public_bytes(client_ltpk)
 
         try:
             verifying_key.verify(client_proof, data)
-        except ed25519.BadSignatureError:
+        except InvalidSignature:
             logger.error("Bad signature, abort.")
             raise
 
@@ -390,7 +391,10 @@ class HAPServerHandler:
             long_to_bytes(session_key), self.PAIRING_5_SALT, self.PAIRING_5_INFO
         )
 
-        server_public = self.state.public_key.to_bytes()
+        server_public = self.state.public_key.public_bytes(
+            encoding=serialization.Encoding.Raw,
+            format=serialization.PublicFormat.Raw,
+        )
         mac = self.state.mac.encode()
 
         material = output_key + mac + server_public
@@ -457,16 +461,21 @@ class HAPServerHandler:
         logger.debug("%s: Pair verify [1/2].", self.client_address)
         client_public = tlv_objects[HAP_TLV_TAGS.PUBLIC_KEY]
 
-        private_key = curve25519.Private()
-        public_key = private_key.get_public()
-        shared_key = private_key.get_shared_key(
-            curve25519.Public(client_public),
-            # Key is hashed before being returned, we don't want it; This fixes that.
-            lambda x: x,
+        private_key = x25519.X25519PrivateKey.generate()
+        public_key = private_key.public_key()
+        shared_key = private_key.exchange(
+            x25519.X25519PublicKey.from_public_bytes(client_public)
         )
 
         mac = self.state.mac.encode()
-        material = public_key.serialize() + mac + client_public
+        material = (
+            public_key.public_bytes(
+                encoding=serialization.Encoding.Raw,
+                format=serialization.PublicFormat.Raw,
+            )
+            + mac
+            + client_public
+        )
         server_proof = self.state.private_key.sign(material)
 
         output_key = hap_hkdf(shared_key, self.PVERIFY_1_SALT, self.PVERIFY_1_INFO)
@@ -487,7 +496,10 @@ class HAPServerHandler:
             HAP_TLV_TAGS.ENCRYPTED_DATA,
             aead_message,
             HAP_TLV_TAGS.PUBLIC_KEY,
-            public_key.serialize(),
+            public_key.public_bytes(
+                encoding=serialization.Encoding.Raw,
+                format=serialization.PublicFormat.Raw,
+            ),
         )
         self._send_tlv_pairing_response(data)
 
@@ -513,7 +525,10 @@ class HAPServerHandler:
         material = (
             self.enc_context["client_public"]
             + client_username
-            + self.enc_context["public_key"].serialize()
+            + self.enc_context["public_key"].public_bytes(
+                encoding=serialization.Encoding.Raw,
+                format=serialization.PublicFormat.Raw,
+            )
         )
 
         client_uuid = uuid.UUID(str(client_username, "utf-8"))
@@ -528,10 +543,10 @@ class HAPServerHandler:
             self._send_authentication_error_tlv_response(HAP_TLV_STATES.M4)
             return
 
-        verifying_key = ed25519.VerifyingKey(perm_client_public)
+        verifying_key = ed25519.Ed25519PublicKey.from_public_bytes(perm_client_public)
         try:
             verifying_key.verify(dec_tlv_objects[HAP_TLV_TAGS.PROOF], material)
-        except ed25519.BadSignatureError:
+        except InvalidSignature:
             logger.error("%s: Bad signature, abort.", self.client_address)
             self._send_authentication_error_tlv_response(HAP_TLV_STATES.M4)
             return
